"""Parse the calculation definitions into a tree
"""
from bs4 import BeautifulSoup
import anytree

path = "domtag/resources/newmarketcorp/neu-20211231_cal.xml"

with open(path, "r") as f:
    soup = BeautifulSoup(f.read(), 'xml')
links = soup.find_all('link:calculationArc')
nodes = {}
for link in links:
    parent_name = link.attrs['xlink:from'].rsplit('_', 1)[0].rsplit('_', 1)[-1]
    child_name = link.attrs['xlink:to'].rsplit('_', 1)[0].rsplit('_', 1)[-1]
    weight = float(link.attrs['weight'])
    if parent_name not in nodes:
        nodes[parent_name] = anytree.Node(parent_name)
    parent_node = nodes[parent_name]
    if child_name not in nodes:
        nodes[child_name] = anytree.Node(child_name)
    child_node = nodes[child_name]
    child_node.parent = parent_node
    child_node.weight = weight

for node in nodes.values():
    if 'weight' in node.__dict__:
        # noinspection PyUnresolvedReferences
        if node.weight == 1.0:
            node.sign = '+'
        elif node.weight == -1.0:
            node.sign = '-'
        else:
            # noinspection PyUnresolvedReferences
            raise Exception(f"Unknown weight {node.weight}")
    else:
        node.sign = ''

for node_name, node in nodes.items():
    if not node.parent and node.children:
        for pre, fill, sub_node in anytree.RenderTree(node):
            print("%s%s %s" % (
                pre,
                sub_node.sign,
                sub_node.name
            ))



"""Common scripts"""

from io import BytesIO
from pathlib import Path
from typing import Optional, Union, Callable, BinaryIO

import pandas as pd
from openpyxl.utils import get_column_letter
from openpyxl.worksheet.worksheet import Worksheet


def adjust_column_width_openpyxl(
        worksheet: Worksheet,
        min_width: int = 15,
        max_width: Optional[int] = 50,
        fixed_width: Optional[int] = None,
) -> None:
    """Adjust column width to fit the content, for a given Excel sheet

    :param worksheet: openpyxl worksheet
    :param min_width: Minimum width of a column, if it is empty
    :param max_width: The maximum width of a column
    :param fixed_width: If this value is provided,
        it will apply to all the columns
    :return: Does not return anything, changes the worksheet inplace
    """
    column_widths = []
    for row in worksheet.values:
        for i, cell in enumerate(row):
            cell_len = min_width
            if len(column_widths) > i:
                if cell is not None and cell != '':
                    cell_len = len(str(cell))
                if cell_len > column_widths[i]:
                    column_widths[i] = cell_len
            else:
                column_widths += [cell_len]
    for i, column_width in enumerate(column_widths, 1):
        width = column_width
        if fixed_width is not None:
            width = fixed_width
        if max_width is not None:
            width = min(width, max_width)
        worksheet.column_dimensions[
            get_column_letter(i)].width = width


def save_to_excel_sheets_output(dfs: dict[str, pd.DataFrame],
                                path: Optional[Union[str, Path]] = None,
                                styling_func: Optional[Callable] = None,
                                styling_func_axis: int = 1,
                                ) -> BinaryIO:
    """Save dictionary of dataframes into an Excel file where each sheet name
    is the corresponding key of the dictionary

    :param dfs: Dictionary, where keys are sheet names and values are DFs
    :param path: String to output path or Path object
    :param styling_func: Optional function for styling the Excel sheets
    :param styling_func_axis: Axis along which the styling is applied
    :return:
    """
    if path is None:
        path = BytesIO()
    elif isinstance(path, str):
        path = Path(path)
    elif not isinstance(path, Path):
        raise TypeError(f"Expected string or Path object, got {type(path)}")
    with pd.ExcelWriter(path, engine='openpyxl') as writer:
        for sheet_name, df in dfs.items():
            if styling_func:
                df.style.apply(
                    styling_func,
                    axis=styling_func_axis).to_excel(excel_writer=writer,
                                                     sheet_name=sheet_name)
            else:
                df.to_excel(excel_writer=writer, sheet_name=sheet_name)
            sheet = writer.sheets[sheet_name]
            adjust_column_width_openpyxl(sheet)
        writer.save()
    return path



from datetime import datetime

from pydantic import BaseModel


class USGAAPAccount(BaseModel):
    """DataClass for US-GAAP taxonomy based account instances"""
    name: str
    context_ref: str
    decimals: str
    id: str
    unit_ref: str  # TODO: change to a dataclass
    value: float
    entity_id: str
    start_date: datetime
    end_date: datetime



from bs4 import BeautifulSoup
import anytree

with open(
        "/domtag/resources/us-gaap-2021-01-31.xsd",
        "r") as f:
    soup = BeautifulSoup(f.read(), 'xml')
links = soup.find_all('link:definitionArc')
nodes = {}
for link in links:
    parent_name = link.attrs['xlink:from'].split('_', 1)[-1]
    child_name = link.attrs['xlink:to'].split('_', 1)[-1]
    if parent_name not in nodes:
        nodes[parent_name] = anytree.Node(parent_name)
    parent_node = nodes[parent_name]
    if child_name not in nodes:
        nodes[child_name] = anytree.Node(child_name)
    child_node = nodes[child_name]
    child_node.parent = parent_node



import enum


class TagAttrs(enum.Enum):
    left_indentation = 'leftIndentation'
    is_italic = 'idItalic'
    is_bold = 'isBold'
    has_top_border = 'hasTopBorder'
    has_bottom_border = 'hasBottomBorder'
